"use client";
import { Answer } from "@/components/search/answer";
import { Relates } from "@/components/search/relates";
import { Sources } from "@/components/search/sources";
import { Relate } from "@/types/relate";
import { Source } from "@/types/source";
import { parseStreaming } from "@/utils/parse-streaming";
import { Annoyed, Mails } from "lucide-react";
import { FC, useEffect, useState } from "react";

export const Result: FC<{ query: string; rid: string, updateQuery: (query: string) => void }> = ({ query, rid, updateQuery }) => {
  const [sources, setSources] = useState<Source[]>([]);
  const [markdown, setMarkdown] = useState<string>("");
  const [relates, setRelates] = useState<Relate[] | null>(null);
  const [error, setError] = useState<number | null>(null);
  const [serverQuery, setServerQuery] = useState("");
  useEffect(() => {
    const controller = new AbortController();
    void parseStreaming(
      controller,
      query,
      rid,
      "query",
      setSources,
      setMarkdown,
      setRelates,
      setServerQuery,
      setError,
    );
    return () => {
      controller.abort();
    };
  }, [query, rid]);

  useEffect(() => {
    updateQuery(serverQuery);
  }, [serverQuery, updateQuery]);

  return (
    <div className="flex flex-col gap-8" >
      <Answer markdown={markdown} sources={sources}></Answer>
      <Sources sources={sources}></Sources>
      <Relates relates={relates}></Relates>
      {error && (
        <div className="absolute inset-4 flex items-center justify-center bg-white/40 backdrop-blur-sm">
          <div className="p-4 bg-white shadow-2xl rounded text-blue-500 font-medium flex gap-4">
            <Annoyed></Annoyed>
            {error === 429
              ? "抱歉，您的请求过于频繁，Vaala被累死了QwQ，请稍后再试吧。"
              : <div className="text-center">{
                error === 400 ?
                  <div>请求不合法，换个关键词试试吧！</div>
                  : (error === 401 ?
                    <div>要不试试登录呢？</div>
                    : (error === 410 ?
                      (true ? <div>内容过期啦，换个关键词再搜一下呢～</div> : <div>对方分享的内容已过期，请登录后换个关键词搜索～</div>)
                      : (<div>服务器出错，Vaala紧急修复中！
                        <div>
                          <a
                            className="text-blue-500 font-medium inline-flex gap-1 items-center flex-nowrap text-nowrap"
                            href="mailto:me@vaala.cat"
                          >
                            <Mails size={8} />
                            点我提交问题
                          </a>
                        </div>
                      </div>)))
              }</div>}
          </div>
        </div>
      )}
    </div>
  );
};
