import { Source } from "@/types/source";
import { Relate } from "@/types/relate";
import { fetchStream } from "@/utils/fetch-stream";

const LLM_SPLIT = "__LLM_RESPONSE__";
const RELATED_SPLIT = "__RELATED_QUESTIONS__";

export const parseStreaming = async (
	controller: AbortController,
	query: string,
	search_uuid: string,
	mode: "query" | "share",
	onSources: (value: Source[]) => void,
	onMarkdown: (value: string) => void,
	onRelates: (value: Relate[]) => void,
	onServerQuery: (value: string) => void,
	onError?: (status: number) => void,
) => {
	const decoder = new TextDecoder();
	let uint8Array = new Uint8Array();
	let chunks = "";
	let sourcesEmitted = false;
	const response = await fetch(mode === "query" ? `/v1/chat/search` : `/v1/chat/share`, {
		method: "POST",
		headers: {
			"Content-Type": "application/json",
			Accept: "*./*",
		},
		signal: controller.signal,
		body: JSON.stringify({
			query,
			search_uuid,
		}),
	});
	if (response.status !== 200) {
		onError?.(response.status);
		return;
	}
	const markdownParse = (text: string) => {
		onMarkdown(
			text
				.replace(/\[\[([cC])itation/g, "[citation")
				.replace(/[cC]itation:(\d+)]]/g, "citation:$1]")
				.replace(/\[\[([cC]itation:\d+)]](?!])/g, `[$1]`)
				.replace(/\[[cC]itation:(\d+)]/g, "[citation]($1)")
				.replace(/\[(\d+)\]/g, "[citation]($1)")
				.replace(/\[引用:(\d+)\]/g, "[citation]($1)")
				.replace(/\[引用(\d+)\]/g, "[citation]($1)"),
		);
	};
	fetchStream(
		response,
		(chunk) => {
			uint8Array = new Uint8Array([...uint8Array, ...chunk]);
			chunks = decoder.decode(uint8Array, { stream: true });
			if (chunks.includes(LLM_SPLIT)) {
				const [sources, rest] = chunks.split(LLM_SPLIT);
				if (!sourcesEmitted) {
					try {
						onServerQuery((JSON.parse(sources)).query);
						onSources((JSON.parse(sources)).contexts);
					} catch (e) {
						onSources([]);
					}
				}
				sourcesEmitted = true;
				if (rest.includes(RELATED_SPLIT)) {
					const [md] = rest.split(RELATED_SPLIT);
					markdownParse(md);
				} else {
					markdownParse(rest);
				}
			}
		},
		() => {
			const [_, relates] = chunks.split(RELATED_SPLIT);
			try {
				onRelates(JSON.parse(relates));
				window.history.replaceState(null, "", "/search?rid=" + search_uuid);
			} catch (e) {
				onRelates([]);
			}
		},
	);
};
